<?php namespace App\Http\Controllers;

use App\Http\Requests;
use App\Http\Controllers\Controller;

use App\Order;
use App\Product;
use App\Day;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;

class AdminController extends Controller {

    public function __construct()
    {
        $this->middleware('admin');
    }

	/**
	 * Display a listing of the resource.
	 *
	 * @return Response
	 */
	public function index()
	{
        // All users except admin
        $userCount = count(User::all())-1;

        // All orders
        $orderCount = count(Order::all());


        return view('admin.overview', compact('userCount','orderCount'));
	}

    public function addProduct()
    {
        return view('admin.addProduct');
    }

    public function deleteProduct($id)
    {
        $product = Product::find($id);

        return view('admin.deleteProduct', compact('product'));
    }

    public function showProducts()
    {
        $products = Product::all();

        return view('admin.showProducts', compact('products'));
    }

    public function redirectAfterAddingNewProduct()
    {
        return redirect('/administrator')->with('message', 'Login Failed');
    }

    public function todayOrderList()
    {
        $currentDate = Carbon::now()->setTime(0,0,0);

        $dayOrderData = User::join('orders','users.user_id', '=', 'orders.user_id')
            ->join('products','orders.product_id','=', 'products.product_id')
            ->join('days','orders.order_id','=', 'days.order_id')
            ->where('days.date', '=', $currentDate)
            ->select('users.user_id',
                'users.first_name',
                'users.last_name',
                'users.note',
                'products.name',
                'days.date',
                'users.note',
                'days.first_meal',
                'days.second_meal',
                'days.third_meal',
                'days.fourth_meal',
                'days.fifth_meal')
            ->get();


        $todayOrderCount = count($dayOrderData);

        $firstMealCount = 0;
        $secondMealCount = 0;
        $thirdMealCount = 0;
        $fourthMealCount = 0;
        $fifthMealCount = 0;
        foreach ($dayOrderData as $data)
        {
             if ($data->first_meal == 1)
             {
                 ++$firstMealCount;
             }
            if ($data->second_meal == 1)
            {
                ++$secondMealCount;
            }

            if ($data->third_meal == 1)
            {
                ++$thirdMealCount;
            }

            if ($data->fourth_meal == 1)
            {
                ++$fourthMealCount;
            }

            if ($data->fifth_meal == 1)
            {
                ++$fifthMealCount;
            }

        }

        return view('admin.todayOrderList', compact('currentDate','dayOrderData', 'todayOrderCount', 'firstMealCount',
            'secondMealCount', 'thirdMealCount', 'fourthMealCount', 'fifthMealCount'));

    }

	/**
	 * Show the form for creating a new resource.
	 *
	 * @return Response
	 */
	public function create()
	{
		//
	}

	/**
	 * Store a newly created resource in storage.
	 *
	 * @return Response
	 */
	public function store()
	{
		//
	}

	/**
	 * Display the specified resource.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function show($id)
	{
		//
	}

	/**
	 * Show the form for editing the specified resource.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function edit($id)
	{
		//
	}

	/**
	 * Update the specified resource in storage.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function update($id)
	{
		//
	}

	/**
	 * Remove the specified resource from storage.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function destroy($id)
	{
		//
	}

}
