<?php namespace App\Http\Controllers;

use App\Http\Requests;
use App\Http\Controllers\Controller;

use App\User;
use Illuminate\Http\Request;
use App\Product;
use App\Order;
use App\Day;
use Carbon\Carbon;
use Illuminate\Validation\Validator;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Redirect;

class OrderController extends Controller {

	/**
	 * Display a listing of the resource.
	 *
	 * @return Response
	 */
	public function index()
	{
        $products = Product::all();
		return view('pages.order', compact('products'));
	}

	/**
	 * Show the page with product order.
	 *
	 * @return Response
	 */
    public function order($id)
    {
       $product = Product::find($id);
        if (Auth::user()) {
            return view('user.productOrder', compact('product'));
        } else {
            \Session::flash('error_message','Pro objednání produktu se, prosím, přihlašte nebo registrujte.');
            return redirect('/');
        }
    }


	/**
	 * Create a new order.
	 *
	 * @return Response
	 */
	public function store($id, Requests\CreateNewOrderRequest $request)
	{
        $startDate = $request->input('start_date');
        $endDate = $request->input('end_date');

        /* If input is not empty a the format is correct. */
        if( ($startDate != '') && ($endDate != '') && (Order::validateDate($startDate)) && (Order::validateDate($endDate))) {

            /* If end date is greater than start date. */
            if (($endDate >= $startDate)) {
                $currentDateTime = Carbon::now();

                /* If business condition is true. */
                if (Order::orderCondition($startDate))
                {
                    Order::createNewOrder($startDate, $endDate, $id, $currentDateTime);
                    \Session::flash('flash_message','Objednávka byla odeslána. Budeme vás kontaktovat.');
                    return redirect('/');
                } else {
                    \Session::flash('error_message','Krabičky na následující den si můžete objednat pouze do 12:00.');
                    return Redirect::back();
                }

            } else {
                \Session::flash('error_message','Vybraný den začátku období nemůže být později než den konce období.');
                return Redirect::back();
            }

        } else {
            \Session::flash('error_message','Zadali jste špatné údaje.');
            return Redirect::back();
        }



	}

    /**
     * Get all user's orders.
     *
     * @return \Illuminate\View\View
     */

    public function myOrders()
    {
        $user = Auth::user();
        $orders = $user->order;

        return view('user.myOrders', compact('user','orders'));
    }

    /**
     * Display all days of the order and set meals to true or false.
     *
     * @return \Illuminate\View\View
     */
    public function editOrder($id)
    {
        $order = Order::find($id);
        $days = $order->getDaysForOrder($order);
        $product = $order->product;

        return view('user.editOrder', compact('order','days', 'product'));
    }


    /**
     * Update meal state.
     *
     * @param $id
     * @return mixed
     */
    public function updateOrderDay(Requests\UpdateOrderDate $request, $id)
    {
        $order = Order::find($id);
        $days = $order->getDaysForOrder($order);

        foreach ($days as $day)
        {
            $firstMeal = $request->has('first_meal' . $day->day_id);
            $secondMeal = $request->has('second_meal' . $day->day_id);
            $thirdMeal = $request->has('third_meal' . $day->day_id);
            $fourthMeal = $request->has('fourth_meal' . $day->day_id);
            $fifthMeal = $request->has('fifth_meal' . $day->day_id);

            $day->first_meal = $firstMeal;
            $day->second_meal = $secondMeal;
            $day->third_meal = $thirdMeal;
            $day->fourth_meal = $fourthMeal;
            $day->fifth_meal = $fifthMeal;

            $day->save();
        }

        \Session::flash('flash_message','Objednávka byla pozměněna.');
        return Redirect::back();
    }




}
