<?php namespace App;

use Illuminate\Auth\Authenticatable;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Auth\Passwords\CanResetPassword;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Contracts\Auth\CanResetPassword as CanResetPasswordContract;

use Auth;

class User extends Model implements AuthenticatableContract, CanResetPasswordContract {

	use Authenticatable, CanResetPassword;

	/**
	 * The database table used by the model.
	 *
	 * @var string
	 */
	protected $table = 'users';
    protected $primaryKey = 'user_id';


	/**
	 * The attributes that are mass assignable.
	 *
	 * @var array
	 */
	protected $fillable = ['first_name', 'last_name', 'email', 'phone', 'password', 'note'];

	/**
	 * The attributes excluded from the model's JSON form.
	 *
	 * @var array
	 */
	protected $hidden = ['password', 'remember_token'];


    public function role()
    {
        return $this->belongsTo('App\Role', 'role_id');
    }

    public function address()
    {
        return $this->belongsTo('App\Address', 'address_id');
    }

    public function order()
    {
        return $this->hasMany('App\Order');
    }

    /**
     * Save a new user after filling out register form.
     *
     * @param array $userAttributes
     * @param array $addressAttributes
     * @return Model|static
     */
    public static function createNewUser(array $userAttributes, array $addressAttributes)
    {

        $newUser = new static($userAttributes);

        // Add "guest" role.
        $guestRole = Role::find(2);
        $newUser = $guestRole->user()->save($newUser);

        // Create new user's address.
        $newAddress = Address::createNewAddress($addressAttributes);
        $newUser = $newAddress->user()->save($newUser);


        return $newUser;
    }

    public function isAdmin()
    {
        $roleId = $this->role_id;

        if ($roleId == 1)
        {
            return true;
        }

        return false;
    }

    /**
     * Return user's address.
     *
     * @param $userID
     * @return mixed
     */
    public static function getUserAddress($userID)
    {
        $user = User::find($userID);
        $address = $user->address;

        return $address->street . " " . $address->house_number . ", " . $address->city . ", " . $address->postal_code;
    }





}
